#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import concrete.uuid.ttypes
import concrete.twitter.ttypes
import concrete.email.ttypes
import concrete.nitf.ttypes


from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class TheoryDependencies(object):
  """
  A struct that holds UUIDs for all theories that a particular
  annotation was based upon (and presumably requires).

  Producers of TheoryDependencies should list all stages that they
  used in constructing their particular annotation. They do not,
  however, need to explicitly label *each* stage; they can label
  only the immediate stage before them.

  Examples:

  If you are producing a Tokenization, and only used the
  SentenceSegmentation in order to produce that Tokenization, list
  only the single SentenceSegmentation UUID in sentenceTheoryList.

  In this example, even though the SentenceSegmentation will have
  a dependency on some SectionSegmentation, it is not necessary
  for the Tokenization to list the SectionSegmentation UUID as a
  dependency.

  If you are a producer of EntityMentions, and you use two
  POSTokenTagging and one NERTokenTagging objects, add the UUIDs for
  the POSTokenTagging objects to posTagTheoryList, and the UUID of
  the NER TokenTagging to the nerTagTheoryList.

  In this example, because multiple annotations influenced the
  new annotation, they should all be listed as dependencies.

  Attributes:
   - sectionTheoryList
   - sentenceTheoryList
   - tokenizationTheoryList
   - posTagTheoryList
   - nerTagTheoryList
   - lemmaTheoryList
   - langIdTheoryList
   - parseTheoryList
   - dependencyParseTheoryList
   - tokenAnnotationTheoryList
   - entityMentionSetTheoryList
   - entitySetTheoryList
   - situationMentionSetTheoryList
   - situationSetTheoryList
   - communicationsList
  """

  thrift_spec = (
    None, # 0
    (1, TType.LIST, 'sectionTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 1
    (2, TType.LIST, 'sentenceTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 2
    (3, TType.LIST, 'tokenizationTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 3
    (4, TType.LIST, 'posTagTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 4
    (5, TType.LIST, 'nerTagTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 5
    (6, TType.LIST, 'lemmaTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 6
    (7, TType.LIST, 'langIdTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 7
    (8, TType.LIST, 'parseTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 8
    (9, TType.LIST, 'dependencyParseTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 9
    (10, TType.LIST, 'tokenAnnotationTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 10
    (11, TType.LIST, 'entityMentionSetTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 11
    (12, TType.LIST, 'entitySetTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 12
    (13, TType.LIST, 'situationMentionSetTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 13
    (14, TType.LIST, 'situationSetTheoryList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 14
    (15, TType.LIST, 'communicationsList', (TType.STRUCT,(concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec)), None, ), # 15
  )

  def __init__(self, sectionTheoryList=None, sentenceTheoryList=None, tokenizationTheoryList=None, posTagTheoryList=None, nerTagTheoryList=None, lemmaTheoryList=None, langIdTheoryList=None, parseTheoryList=None, dependencyParseTheoryList=None, tokenAnnotationTheoryList=None, entityMentionSetTheoryList=None, entitySetTheoryList=None, situationMentionSetTheoryList=None, situationSetTheoryList=None, communicationsList=None,):
    self.sectionTheoryList = sectionTheoryList
    self.sentenceTheoryList = sentenceTheoryList
    self.tokenizationTheoryList = tokenizationTheoryList
    self.posTagTheoryList = posTagTheoryList
    self.nerTagTheoryList = nerTagTheoryList
    self.lemmaTheoryList = lemmaTheoryList
    self.langIdTheoryList = langIdTheoryList
    self.parseTheoryList = parseTheoryList
    self.dependencyParseTheoryList = dependencyParseTheoryList
    self.tokenAnnotationTheoryList = tokenAnnotationTheoryList
    self.entityMentionSetTheoryList = entityMentionSetTheoryList
    self.entitySetTheoryList = entitySetTheoryList
    self.situationMentionSetTheoryList = situationMentionSetTheoryList
    self.situationSetTheoryList = situationSetTheoryList
    self.communicationsList = communicationsList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.sectionTheoryList = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = concrete.uuid.ttypes.UUID()
            _elem5.read(iprot)
            self.sectionTheoryList.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.LIST:
          self.sentenceTheoryList = []
          (_etype9, _size6) = iprot.readListBegin()
          for _i10 in xrange(_size6):
            _elem11 = concrete.uuid.ttypes.UUID()
            _elem11.read(iprot)
            self.sentenceTheoryList.append(_elem11)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.LIST:
          self.tokenizationTheoryList = []
          (_etype15, _size12) = iprot.readListBegin()
          for _i16 in xrange(_size12):
            _elem17 = concrete.uuid.ttypes.UUID()
            _elem17.read(iprot)
            self.tokenizationTheoryList.append(_elem17)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.LIST:
          self.posTagTheoryList = []
          (_etype21, _size18) = iprot.readListBegin()
          for _i22 in xrange(_size18):
            _elem23 = concrete.uuid.ttypes.UUID()
            _elem23.read(iprot)
            self.posTagTheoryList.append(_elem23)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.nerTagTheoryList = []
          (_etype27, _size24) = iprot.readListBegin()
          for _i28 in xrange(_size24):
            _elem29 = concrete.uuid.ttypes.UUID()
            _elem29.read(iprot)
            self.nerTagTheoryList.append(_elem29)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.LIST:
          self.lemmaTheoryList = []
          (_etype33, _size30) = iprot.readListBegin()
          for _i34 in xrange(_size30):
            _elem35 = concrete.uuid.ttypes.UUID()
            _elem35.read(iprot)
            self.lemmaTheoryList.append(_elem35)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.LIST:
          self.langIdTheoryList = []
          (_etype39, _size36) = iprot.readListBegin()
          for _i40 in xrange(_size36):
            _elem41 = concrete.uuid.ttypes.UUID()
            _elem41.read(iprot)
            self.langIdTheoryList.append(_elem41)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.LIST:
          self.parseTheoryList = []
          (_etype45, _size42) = iprot.readListBegin()
          for _i46 in xrange(_size42):
            _elem47 = concrete.uuid.ttypes.UUID()
            _elem47.read(iprot)
            self.parseTheoryList.append(_elem47)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 9:
        if ftype == TType.LIST:
          self.dependencyParseTheoryList = []
          (_etype51, _size48) = iprot.readListBegin()
          for _i52 in xrange(_size48):
            _elem53 = concrete.uuid.ttypes.UUID()
            _elem53.read(iprot)
            self.dependencyParseTheoryList.append(_elem53)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 10:
        if ftype == TType.LIST:
          self.tokenAnnotationTheoryList = []
          (_etype57, _size54) = iprot.readListBegin()
          for _i58 in xrange(_size54):
            _elem59 = concrete.uuid.ttypes.UUID()
            _elem59.read(iprot)
            self.tokenAnnotationTheoryList.append(_elem59)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 11:
        if ftype == TType.LIST:
          self.entityMentionSetTheoryList = []
          (_etype63, _size60) = iprot.readListBegin()
          for _i64 in xrange(_size60):
            _elem65 = concrete.uuid.ttypes.UUID()
            _elem65.read(iprot)
            self.entityMentionSetTheoryList.append(_elem65)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 12:
        if ftype == TType.LIST:
          self.entitySetTheoryList = []
          (_etype69, _size66) = iprot.readListBegin()
          for _i70 in xrange(_size66):
            _elem71 = concrete.uuid.ttypes.UUID()
            _elem71.read(iprot)
            self.entitySetTheoryList.append(_elem71)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 13:
        if ftype == TType.LIST:
          self.situationMentionSetTheoryList = []
          (_etype75, _size72) = iprot.readListBegin()
          for _i76 in xrange(_size72):
            _elem77 = concrete.uuid.ttypes.UUID()
            _elem77.read(iprot)
            self.situationMentionSetTheoryList.append(_elem77)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 14:
        if ftype == TType.LIST:
          self.situationSetTheoryList = []
          (_etype81, _size78) = iprot.readListBegin()
          for _i82 in xrange(_size78):
            _elem83 = concrete.uuid.ttypes.UUID()
            _elem83.read(iprot)
            self.situationSetTheoryList.append(_elem83)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 15:
        if ftype == TType.LIST:
          self.communicationsList = []
          (_etype87, _size84) = iprot.readListBegin()
          for _i88 in xrange(_size84):
            _elem89 = concrete.uuid.ttypes.UUID()
            _elem89.read(iprot)
            self.communicationsList.append(_elem89)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TheoryDependencies')
    if self.sectionTheoryList is not None:
      oprot.writeFieldBegin('sectionTheoryList', TType.LIST, 1)
      oprot.writeListBegin(TType.STRUCT, len(self.sectionTheoryList))
      for iter90 in self.sectionTheoryList:
        iter90.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.sentenceTheoryList is not None:
      oprot.writeFieldBegin('sentenceTheoryList', TType.LIST, 2)
      oprot.writeListBegin(TType.STRUCT, len(self.sentenceTheoryList))
      for iter91 in self.sentenceTheoryList:
        iter91.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.tokenizationTheoryList is not None:
      oprot.writeFieldBegin('tokenizationTheoryList', TType.LIST, 3)
      oprot.writeListBegin(TType.STRUCT, len(self.tokenizationTheoryList))
      for iter92 in self.tokenizationTheoryList:
        iter92.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.posTagTheoryList is not None:
      oprot.writeFieldBegin('posTagTheoryList', TType.LIST, 4)
      oprot.writeListBegin(TType.STRUCT, len(self.posTagTheoryList))
      for iter93 in self.posTagTheoryList:
        iter93.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.nerTagTheoryList is not None:
      oprot.writeFieldBegin('nerTagTheoryList', TType.LIST, 5)
      oprot.writeListBegin(TType.STRUCT, len(self.nerTagTheoryList))
      for iter94 in self.nerTagTheoryList:
        iter94.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.lemmaTheoryList is not None:
      oprot.writeFieldBegin('lemmaTheoryList', TType.LIST, 6)
      oprot.writeListBegin(TType.STRUCT, len(self.lemmaTheoryList))
      for iter95 in self.lemmaTheoryList:
        iter95.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.langIdTheoryList is not None:
      oprot.writeFieldBegin('langIdTheoryList', TType.LIST, 7)
      oprot.writeListBegin(TType.STRUCT, len(self.langIdTheoryList))
      for iter96 in self.langIdTheoryList:
        iter96.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.parseTheoryList is not None:
      oprot.writeFieldBegin('parseTheoryList', TType.LIST, 8)
      oprot.writeListBegin(TType.STRUCT, len(self.parseTheoryList))
      for iter97 in self.parseTheoryList:
        iter97.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.dependencyParseTheoryList is not None:
      oprot.writeFieldBegin('dependencyParseTheoryList', TType.LIST, 9)
      oprot.writeListBegin(TType.STRUCT, len(self.dependencyParseTheoryList))
      for iter98 in self.dependencyParseTheoryList:
        iter98.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.tokenAnnotationTheoryList is not None:
      oprot.writeFieldBegin('tokenAnnotationTheoryList', TType.LIST, 10)
      oprot.writeListBegin(TType.STRUCT, len(self.tokenAnnotationTheoryList))
      for iter99 in self.tokenAnnotationTheoryList:
        iter99.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.entityMentionSetTheoryList is not None:
      oprot.writeFieldBegin('entityMentionSetTheoryList', TType.LIST, 11)
      oprot.writeListBegin(TType.STRUCT, len(self.entityMentionSetTheoryList))
      for iter100 in self.entityMentionSetTheoryList:
        iter100.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.entitySetTheoryList is not None:
      oprot.writeFieldBegin('entitySetTheoryList', TType.LIST, 12)
      oprot.writeListBegin(TType.STRUCT, len(self.entitySetTheoryList))
      for iter101 in self.entitySetTheoryList:
        iter101.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.situationMentionSetTheoryList is not None:
      oprot.writeFieldBegin('situationMentionSetTheoryList', TType.LIST, 13)
      oprot.writeListBegin(TType.STRUCT, len(self.situationMentionSetTheoryList))
      for iter102 in self.situationMentionSetTheoryList:
        iter102.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.situationSetTheoryList is not None:
      oprot.writeFieldBegin('situationSetTheoryList', TType.LIST, 14)
      oprot.writeListBegin(TType.STRUCT, len(self.situationSetTheoryList))
      for iter103 in self.situationSetTheoryList:
        iter103.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.communicationsList is not None:
      oprot.writeFieldBegin('communicationsList', TType.LIST, 15)
      oprot.writeListBegin(TType.STRUCT, len(self.communicationsList))
      for iter104 in self.communicationsList:
        iter104.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.sectionTheoryList)
    value = (value * 31) ^ hash(self.sentenceTheoryList)
    value = (value * 31) ^ hash(self.tokenizationTheoryList)
    value = (value * 31) ^ hash(self.posTagTheoryList)
    value = (value * 31) ^ hash(self.nerTagTheoryList)
    value = (value * 31) ^ hash(self.lemmaTheoryList)
    value = (value * 31) ^ hash(self.langIdTheoryList)
    value = (value * 31) ^ hash(self.parseTheoryList)
    value = (value * 31) ^ hash(self.dependencyParseTheoryList)
    value = (value * 31) ^ hash(self.tokenAnnotationTheoryList)
    value = (value * 31) ^ hash(self.entityMentionSetTheoryList)
    value = (value * 31) ^ hash(self.entitySetTheoryList)
    value = (value * 31) ^ hash(self.situationMentionSetTheoryList)
    value = (value * 31) ^ hash(self.situationSetTheoryList)
    value = (value * 31) ^ hash(self.communicationsList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Digest(object):
  """
  Analytic-specific information about an attribute or edge. Digests
  are used to combine information from multiple sources to generate a
  unified value. The digests generated by an analytic will only ever
  be used by that same analytic, so analytics can feel free to encode
  information in whatever way is convenient.

  Attributes:
   - bytesValue: The following fields define various ways you can store the
  digest data (for convenience). If none of these meets your
  needs, then serialize the digest to a byte sequence and store it
  in bytesValue.
   - int64Value
   - doubleValue
   - stringValue
   - int64List
   - doubleList
   - stringList
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'bytesValue', None, None, ), # 1
    (2, TType.I64, 'int64Value', None, None, ), # 2
    (3, TType.DOUBLE, 'doubleValue', None, None, ), # 3
    (4, TType.STRING, 'stringValue', None, None, ), # 4
    (5, TType.LIST, 'int64List', (TType.I64,None), None, ), # 5
    (6, TType.LIST, 'doubleList', (TType.DOUBLE,None), None, ), # 6
    (7, TType.LIST, 'stringList', (TType.STRING,None), None, ), # 7
  )

  def __init__(self, bytesValue=None, int64Value=None, doubleValue=None, stringValue=None, int64List=None, doubleList=None, stringList=None,):
    self.bytesValue = bytesValue
    self.int64Value = int64Value
    self.doubleValue = doubleValue
    self.stringValue = stringValue
    self.int64List = int64List
    self.doubleList = doubleList
    self.stringList = stringList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.bytesValue = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.int64Value = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.DOUBLE:
          self.doubleValue = iprot.readDouble();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.stringValue = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.int64List = []
          (_etype108, _size105) = iprot.readListBegin()
          for _i109 in xrange(_size105):
            _elem110 = iprot.readI64();
            self.int64List.append(_elem110)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.LIST:
          self.doubleList = []
          (_etype114, _size111) = iprot.readListBegin()
          for _i115 in xrange(_size111):
            _elem116 = iprot.readDouble();
            self.doubleList.append(_elem116)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.LIST:
          self.stringList = []
          (_etype120, _size117) = iprot.readListBegin()
          for _i121 in xrange(_size117):
            _elem122 = iprot.readString().decode('utf-8')
            self.stringList.append(_elem122)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Digest')
    if self.bytesValue is not None:
      oprot.writeFieldBegin('bytesValue', TType.STRING, 1)
      oprot.writeString(self.bytesValue)
      oprot.writeFieldEnd()
    if self.int64Value is not None:
      oprot.writeFieldBegin('int64Value', TType.I64, 2)
      oprot.writeI64(self.int64Value)
      oprot.writeFieldEnd()
    if self.doubleValue is not None:
      oprot.writeFieldBegin('doubleValue', TType.DOUBLE, 3)
      oprot.writeDouble(self.doubleValue)
      oprot.writeFieldEnd()
    if self.stringValue is not None:
      oprot.writeFieldBegin('stringValue', TType.STRING, 4)
      oprot.writeString(self.stringValue.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.int64List is not None:
      oprot.writeFieldBegin('int64List', TType.LIST, 5)
      oprot.writeListBegin(TType.I64, len(self.int64List))
      for iter123 in self.int64List:
        oprot.writeI64(iter123)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.doubleList is not None:
      oprot.writeFieldBegin('doubleList', TType.LIST, 6)
      oprot.writeListBegin(TType.DOUBLE, len(self.doubleList))
      for iter124 in self.doubleList:
        oprot.writeDouble(iter124)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.stringList is not None:
      oprot.writeFieldBegin('stringList', TType.LIST, 7)
      oprot.writeListBegin(TType.STRING, len(self.stringList))
      for iter125 in self.stringList:
        oprot.writeString(iter125.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.bytesValue)
    value = (value * 31) ^ hash(self.int64Value)
    value = (value * 31) ^ hash(self.doubleValue)
    value = (value * 31) ^ hash(self.stringValue)
    value = (value * 31) ^ hash(self.int64List)
    value = (value * 31) ^ hash(self.doubleList)
    value = (value * 31) ^ hash(self.stringList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class AnnotationMetadata(object):
  """
  Metadata associated with an annotation or a set of annotations,
  that identifies where those annotations came from.

  Attributes:
   - tool: The name of the tool that generated this annotation.
   - timestamp: The time at which this annotation was generated (in unix time
  UTC -- i.e., seconds since January 1, 1970).
   - digest: A Digest, carrying over any information the annotation metadata
  wishes to carry over.
   - dependencies: The theories that supported this annotation.

  An empty field indicates that the theory has no
  dependencies (e.g., an ingester).
   - kBest: An integer that represents a ranking for systems
  that output k-best lists.

  For systems that do not output k-best lists,
  the default value (1) should suffice.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'tool', None, None, ), # 1
    (2, TType.I64, 'timestamp', None, None, ), # 2
    None, # 3
    (4, TType.STRUCT, 'digest', (Digest, Digest.thrift_spec), None, ), # 4
    (5, TType.STRUCT, 'dependencies', (TheoryDependencies, TheoryDependencies.thrift_spec), None, ), # 5
    (6, TType.I32, 'kBest', None, 1, ), # 6
  )

  def __init__(self, tool=None, timestamp=None, digest=None, dependencies=None, kBest=thrift_spec[6][4],):
    self.tool = tool
    self.timestamp = timestamp
    self.digest = digest
    self.dependencies = dependencies
    self.kBest = kBest

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.tool = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.timestamp = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRUCT:
          self.digest = Digest()
          self.digest.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRUCT:
          self.dependencies = TheoryDependencies()
          self.dependencies.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.I32:
          self.kBest = iprot.readI32();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('AnnotationMetadata')
    if self.tool is not None:
      oprot.writeFieldBegin('tool', TType.STRING, 1)
      oprot.writeString(self.tool.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.timestamp is not None:
      oprot.writeFieldBegin('timestamp', TType.I64, 2)
      oprot.writeI64(self.timestamp)
      oprot.writeFieldEnd()
    if self.digest is not None:
      oprot.writeFieldBegin('digest', TType.STRUCT, 4)
      self.digest.write(oprot)
      oprot.writeFieldEnd()
    if self.dependencies is not None:
      oprot.writeFieldBegin('dependencies', TType.STRUCT, 5)
      self.dependencies.write(oprot)
      oprot.writeFieldEnd()
    if self.kBest is not None:
      oprot.writeFieldBegin('kBest', TType.I32, 6)
      oprot.writeI32(self.kBest)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.tool is None:
      raise TProtocol.TProtocolException(message='Required field tool is unset!')
    if self.timestamp is None:
      raise TProtocol.TProtocolException(message='Required field timestamp is unset!')
    if self.kBest is None:
      raise TProtocol.TProtocolException(message='Required field kBest is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.tool)
    value = (value * 31) ^ hash(self.timestamp)
    value = (value * 31) ^ hash(self.digest)
    value = (value * 31) ^ hash(self.dependencies)
    value = (value * 31) ^ hash(self.kBest)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class CommunicationMetadata(object):
  """
  Metadata specific to a particular Communication object.
  This might include corpus-specific metadata (from the Twitter API),
  attributes associated with the Communication (the author),
  or other information about the Communication.

  Attributes:
   - tweetInfo: Extra information for communications where kind==TWEET:
  Information about this tweet that is provided by the Twitter
  API.  For information about the Twitter API, see:
  <https://dev.twitter.com/docs/platform-objects>
   - emailInfo: Extra information for communications where kind==EMAIL
   - nitfInfo: Extra information that may come from the NITF
  (News Industry Text Format) schema. See 'nitf.thrift'.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'tweetInfo', (concrete.twitter.ttypes.TweetInfo, concrete.twitter.ttypes.TweetInfo.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'emailInfo', (concrete.email.ttypes.EmailCommunicationInfo, concrete.email.ttypes.EmailCommunicationInfo.thrift_spec), None, ), # 2
    (3, TType.STRUCT, 'nitfInfo', (concrete.nitf.ttypes.NITFInfo, concrete.nitf.ttypes.NITFInfo.thrift_spec), None, ), # 3
  )

  def __init__(self, tweetInfo=None, emailInfo=None, nitfInfo=None,):
    self.tweetInfo = tweetInfo
    self.emailInfo = emailInfo
    self.nitfInfo = nitfInfo

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.tweetInfo = concrete.twitter.ttypes.TweetInfo()
          self.tweetInfo.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.emailInfo = concrete.email.ttypes.EmailCommunicationInfo()
          self.emailInfo.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.nitfInfo = concrete.nitf.ttypes.NITFInfo()
          self.nitfInfo.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('CommunicationMetadata')
    if self.tweetInfo is not None:
      oprot.writeFieldBegin('tweetInfo', TType.STRUCT, 1)
      self.tweetInfo.write(oprot)
      oprot.writeFieldEnd()
    if self.emailInfo is not None:
      oprot.writeFieldBegin('emailInfo', TType.STRUCT, 2)
      self.emailInfo.write(oprot)
      oprot.writeFieldEnd()
    if self.nitfInfo is not None:
      oprot.writeFieldBegin('nitfInfo', TType.STRUCT, 3)
      self.nitfInfo.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.tweetInfo)
    value = (value * 31) ^ hash(self.emailInfo)
    value = (value * 31) ^ hash(self.nitfInfo)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import concrete.uuid.ttypes
import concrete.metadata.ttypes


from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class ClusterMember(object):
  """
  A member of a 'cluster'.

  Contains a communication ID, an Entity|Situation(Mention)Set ID,
  and an element ID that identifies the specific
  Entity|Situation(Mention) associated with this object.

  Attributes:
   - communicationId: The UUID of the Communication this ClusterMember is associated with.
   - setId: The UUID of the Entity|Situation(Mention)Set this ClusterMember
  is associated with.
   - elementId: The UUID of the specific EntityMention, Entity, SituationMention,
  or Situation that this ClusterMember is tied to.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'communicationId', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'setId', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 2
    (3, TType.STRUCT, 'elementId', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 3
  )

  def __init__(self, communicationId=None, setId=None, elementId=None,):
    self.communicationId = communicationId
    self.setId = setId
    self.elementId = elementId

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.communicationId = concrete.uuid.ttypes.UUID()
          self.communicationId.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.setId = concrete.uuid.ttypes.UUID()
          self.setId.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.elementId = concrete.uuid.ttypes.UUID()
          self.elementId.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ClusterMember')
    if self.communicationId is not None:
      oprot.writeFieldBegin('communicationId', TType.STRUCT, 1)
      self.communicationId.write(oprot)
      oprot.writeFieldEnd()
    if self.setId is not None:
      oprot.writeFieldBegin('setId', TType.STRUCT, 2)
      self.setId.write(oprot)
      oprot.writeFieldEnd()
    if self.elementId is not None:
      oprot.writeFieldBegin('elementId', TType.STRUCT, 3)
      self.elementId.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.communicationId is None:
      raise TProtocol.TProtocolException(message='Required field communicationId is unset!')
    if self.setId is None:
      raise TProtocol.TProtocolException(message='Required field setId is unset!')
    if self.elementId is None:
      raise TProtocol.TProtocolException(message='Required field elementId is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.communicationId)
    value = (value * 31) ^ hash(self.setId)
    value = (value * 31) ^ hash(self.elementId)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Cluster(object):
  """
  A structure that represents a cluster of Communications.

  This structure allows for  specific grouping of ClusterMembers,
  as well as other Clusters that this Cluster object considers children.

  Attributes:
   - clusterMemberIndexList: A list of integers that represent indices into a list of ClusterMember
  objects (likely represented in a Clustering object).
   - confidenceList: Confidence values for members of 'memberIndexList' field.
   - childIndexList: A list of integers that represent indices into a list of Cluster objects,
  used to represent child Clusters associated with this particular Cluster.
  """

  thrift_spec = (
    None, # 0
    (1, TType.LIST, 'clusterMemberIndexList', (TType.I32,None), None, ), # 1
    (2, TType.LIST, 'confidenceList', (TType.DOUBLE,None), None, ), # 2
    (3, TType.LIST, 'childIndexList', (TType.I32,None), None, ), # 3
  )

  def __init__(self, clusterMemberIndexList=None, confidenceList=None, childIndexList=None,):
    self.clusterMemberIndexList = clusterMemberIndexList
    self.confidenceList = confidenceList
    self.childIndexList = childIndexList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.clusterMemberIndexList = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = iprot.readI32();
            self.clusterMemberIndexList.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.LIST:
          self.confidenceList = []
          (_etype9, _size6) = iprot.readListBegin()
          for _i10 in xrange(_size6):
            _elem11 = iprot.readDouble();
            self.confidenceList.append(_elem11)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.LIST:
          self.childIndexList = []
          (_etype15, _size12) = iprot.readListBegin()
          for _i16 in xrange(_size12):
            _elem17 = iprot.readI32();
            self.childIndexList.append(_elem17)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Cluster')
    if self.clusterMemberIndexList is not None:
      oprot.writeFieldBegin('clusterMemberIndexList', TType.LIST, 1)
      oprot.writeListBegin(TType.I32, len(self.clusterMemberIndexList))
      for iter18 in self.clusterMemberIndexList:
        oprot.writeI32(iter18)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.confidenceList is not None:
      oprot.writeFieldBegin('confidenceList', TType.LIST, 2)
      oprot.writeListBegin(TType.DOUBLE, len(self.confidenceList))
      for iter19 in self.confidenceList:
        oprot.writeDouble(iter19)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.childIndexList is not None:
      oprot.writeFieldBegin('childIndexList', TType.LIST, 3)
      oprot.writeListBegin(TType.I32, len(self.childIndexList))
      for iter20 in self.childIndexList:
        oprot.writeI32(iter20)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.clusterMemberIndexList)
    value = (value * 31) ^ hash(self.confidenceList)
    value = (value * 31) ^ hash(self.childIndexList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Clustering(object):
  """
  A structure that represents a "clustering" of Communications, to support
  cross-document coreference tasks.

  Attributes:
   - uuid: UUID for this Clustering object.
   - metadata: Metadata for this Clustering object.
   - clusterMemberList: A complete list of ClusterMember objects that this Clustering holds.
   - clusterList: A complete list of Cluster objects that this Clustering holds.
   - rootClusterIndexList: A list of integers that represent root Clusters. These integers should be
  indices into the 'clusterList' object.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'uuid', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'metadata', (concrete.metadata.ttypes.AnnotationMetadata, concrete.metadata.ttypes.AnnotationMetadata.thrift_spec), None, ), # 2
    (3, TType.LIST, 'clusterMemberList', (TType.STRUCT,(ClusterMember, ClusterMember.thrift_spec)), None, ), # 3
    (4, TType.LIST, 'clusterList', (TType.STRUCT,(Cluster, Cluster.thrift_spec)), None, ), # 4
    (5, TType.LIST, 'rootClusterIndexList', (TType.I32,None), None, ), # 5
  )

  def __init__(self, uuid=None, metadata=None, clusterMemberList=None, clusterList=None, rootClusterIndexList=None,):
    self.uuid = uuid
    self.metadata = metadata
    self.clusterMemberList = clusterMemberList
    self.clusterList = clusterList
    self.rootClusterIndexList = rootClusterIndexList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.uuid = concrete.uuid.ttypes.UUID()
          self.uuid.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
          self.metadata.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.LIST:
          self.clusterMemberList = []
          (_etype24, _size21) = iprot.readListBegin()
          for _i25 in xrange(_size21):
            _elem26 = ClusterMember()
            _elem26.read(iprot)
            self.clusterMemberList.append(_elem26)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.LIST:
          self.clusterList = []
          (_etype30, _size27) = iprot.readListBegin()
          for _i31 in xrange(_size27):
            _elem32 = Cluster()
            _elem32.read(iprot)
            self.clusterList.append(_elem32)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.rootClusterIndexList = []
          (_etype36, _size33) = iprot.readListBegin()
          for _i37 in xrange(_size33):
            _elem38 = iprot.readI32();
            self.rootClusterIndexList.append(_elem38)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Clustering')
    if self.uuid is not None:
      oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
      self.uuid.write(oprot)
      oprot.writeFieldEnd()
    if self.metadata is not None:
      oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
      self.metadata.write(oprot)
      oprot.writeFieldEnd()
    if self.clusterMemberList is not None:
      oprot.writeFieldBegin('clusterMemberList', TType.LIST, 3)
      oprot.writeListBegin(TType.STRUCT, len(self.clusterMemberList))
      for iter39 in self.clusterMemberList:
        iter39.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.clusterList is not None:
      oprot.writeFieldBegin('clusterList', TType.LIST, 4)
      oprot.writeListBegin(TType.STRUCT, len(self.clusterList))
      for iter40 in self.clusterList:
        iter40.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.rootClusterIndexList is not None:
      oprot.writeFieldBegin('rootClusterIndexList', TType.LIST, 5)
      oprot.writeListBegin(TType.I32, len(self.rootClusterIndexList))
      for iter41 in self.rootClusterIndexList:
        oprot.writeI32(iter41)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.uuid is None:
      raise TProtocol.TProtocolException(message='Required field uuid is unset!')
    if self.metadata is None:
      raise TProtocol.TProtocolException(message='Required field metadata is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.uuid)
    value = (value * 31) ^ hash(self.metadata)
    value = (value * 31) ^ hash(self.clusterMemberList)
    value = (value * 31) ^ hash(self.clusterList)
    value = (value * 31) ^ hash(self.rootClusterIndexList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



# -*- coding: utf-8 -*-

# The 'coding' line above was manually added to the autogenerated code
# to prevent the error message on import:
#   SyntaxError: Non-ASCII character '\xef' in file concrete/nitf/ttypes.py

#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class NITFInfo(object):
  """
  Attributes:
   - alternateURL: This field specifies the URL of the article, if published online. In some
  cases, such as with the New York Times, when this field is present,
  the URL is preferred to the URL field on articles published on
  or after April 02, 2006, as the linked page will have richer content.
   - articleAbstract: This field is a summary of the article, possibly written by
  an indexing service.
   - authorBiography: This field specifies the biography of the author of the article.
  Generally, this field is specified for guest authors, and not for
  regular reporters, except to provide the author's email address.
   - banner: The banner field is used to indicate if there has been additional
  information appended to the articles since its publication. Examples of
  banners include ('Correction Appended' and 'Editor's Note Appended').
   - biographicalCategoryList: When present, the biographical category field generally indicates that a
  document focuses on a particular individual. The value of the field
  indicates the area or category in which this individual is best known.
  This field is most often defined for Obituaries and Book Reviews.

  <ol>
  <li>Politics and Government (U.S.)</li>
  <li>Books and Magazines <li>Royalty</li>
  </ol>
   - columnName: If the article is part of a regular column, this field specifies the name
  of that column.
  <br />
  Sample Column Names:
  <br />
  <ol>
  <li>World News Briefs</li>
  <li>WEDDINGS</li>
  <li>The Accessories Channel</li>
  </ol>

   - columnNumber: This field specifies the column in which the article starts in the print
  paper. A typical printed page in the paper has six columns numbered from
  right to left. As a consequence most, but not all, of the values for this
  field fall in the range 1-6.
   - correctionDate: This field specifies the date on which a correction was made to the
  article. Generally, if the correction date is specified, the correction
  text will also be specified (and vice versa).
   - correctionText: For articles corrected following publication, this field specifies the
  correction. Generally, if the correction text is specified, the
  correction date will also be specified (and vice versa).
   - credit: This field indicates the entity that produced the editorial content of
  this document.
   - dayOfWeek: This field specifies the day of week on which the article was published.
  <ul>
  <li>Monday</li>
  <li>Tuesday</li>
  <li>Wednesday</li>
  <li>Thursday</li>
  <li>Friday</li>
  <li>Saturday</li>
  <li>Sunday</li>
  </ul>
   - descriptorList: The &quot;descriptors&quot; field specifies a list of descriptive terms drawn from
  a normalized controlled vocabulary corresponding to subjects mentioned in
  the article.
  <br />
  Examples Include:
  <ol>
  <li>ECONOMIC CONDITIONS AND TRENDS</li>
  <li>AIRPLANES</li>
  <li>VIOLINS</li>
  </ol>
   - featurePage: The feature page containing this article, such as
  <ul>
  <li>Education Page</li>
  <li>Fashion Page</li>
  </ul>
   - generalOnlineDescriptorList: The &quot;general online descriptors&quot; field specifies a list of descriptors
  that are at a higher level of generality than the other tags associated
  with the article.
  <br />
  Examples Include:
  <ol>
  <li>Surfing</li>
  <li>Venice Biennale</li>
  <li>Ranches</li>
  </ol>
   - guid: The GUID field specifies an integer that is guaranteed to be unique for
  every document in the corpus.
   - kicker: The kicker is an additional piece of information printed as an
  accompaniment to a news headline.
   - leadParagraphList: The &quot;lead Paragraph&quot; field is the lead paragraph of the article.
  Generally this field is populated with the first two paragraphs from the
  article.
   - locationList: The &quot;locations&quot; field specifies a list of geographic descriptors drawn
  from a normalized controlled vocabulary that correspond to places
  mentioned in the article.
  <br />
  Examples Include:
  <ol>
  <li>Wellsboro (Pa)</li>
  <li>Kansas City (Kan)</li>
  <li>Park Slope (NYC)</li>
  </ol>
   - nameList: The &quot;names&quot; field specifies a list of names mentioned in the article.
  <br />
  Examples Include:
  <ol>
  <li>Azza Fahmy</li>
  <li>George C. Izenour</li>
  <li>Chris Schenkel</li>
  </ol>
   - newsDesk: This field specifies the desk in the newsroom that
  produced the article. The desk is related to, but is not the same as the
  section in which the article appears.
   - normalizedByline: The Normalized Byline field is the byline normalized to the form (last
  name, first name).
   - onlineDescriptorList: This field specifies a list of descriptors from a normalized controlled
  vocabulary that correspond to topics mentioned in the article.
  <br />
  Examples Include:
  <ol>
  <li>Marriages</li>
  <li>Parks and Other Recreation Areas</li>
  <li>Cooking and Cookbooks</li>
  </ol>
   - onlineHeadline: This field specifies the headline displayed with the article
  online. Often this differs from the headline used in print.
   - onlineLeadParagraph: This field specifies the lead paragraph for the online version.
   - onlineLocationList: This field specifies a list of place names that correspond to geographic
  locations mentioned in the article.
  <br />
  Examples Include:
  <ol>
  <li>Hollywood</li>
  <li>Los Angeles</li>
  <li>Arcadia</li>
  </ol>
   - onlineOrganizationList: This field specifies a list of organizations that correspond to
  organizations mentioned in the article.
  <br />
  Examples Include:
  <ol>
  <li>Nintendo Company Limited</li>
  <li>Yeshiva University</li>
  <li>Rose Center</li>
  </ol>
   - onlinePeople: This field specifies a list of people that correspond to individuals
  mentioned in the article.
  <br/>
  Examples Include:
  <ol>
  <li>Lopez, Jennifer</li>
  <li>Joyce, James</li>
  <li>Robinson, Jackie</li>
  </ol>
   - onlineSectionList: This field specifies the section(s) in which the online version of the article
  is placed. This may typically be populated from a semicolon (;) delineated list.
   - onlineTitleList: This field specifies a list of authored works mentioned in the article.
  <br/>
  Examples Include:
  <ol>
  <li>Matchstick Men (Movie)</li>
  <li>Blades of Glory (Movie)</li>
  <li>Bridge & Tunnel (Play)</li>
  </ol>
   - organizationList: This field specifies a list of organization names drawn from a normalized
  controlled vocabulary that correspond to organizations mentioned in the
  article.
  <br />
  Examples Include:
  <ol>
  <li>Circuit City Stores Inc</li>
  <li>Delaware County Community College (Pa)</li>
  <li>CONNECTICUT GRAND OPERA</li>
  </ol>
   - page: This field specifies the page of the section in the paper in which the
  article appears. This is not an absolute pagination. An article that
  appears on page 3 in section A occurs in the physical paper before an
  article that occurs on page 1 of section F. The section is encoded in
  the <strong>section</strong> field.
   - peopleList: This field specifies a list of people from a normalized controlled
  vocabulary that correspond to individuals mentioned in the article.
  <br />
  Examples Include:
  <ol>
  <li>REAGAN, RONALD WILSON (PRES)</li>
  <li>BEGIN, MENACHEM (PRIME MIN)</li>
  <li>COLLINS, GLENN</li>
  </ol>
   - publicationDate: This field specifies the date of the article's publication.
   - publicationDayOfMonth: This field specifies the day of the month on which the article was
  published, always in the range 1-31.
   - publicationMonth: This field specifies the month on which the article was published in the
  range 1-12 where 1 is January 2 is February etc.
   - publicationYear: This field specifies the year in which the article was published. This
  value is in the range 1987-2007 for this collection.
   - section: This field specifies the section of the paper in which the article
  appears. This is not the name of the section, but rather a letter or
  number that indicates the section.
   - seriesName: If the article is part of a regular series, this field specifies the name
  of that column.
   - slug: The slug is a short string that uniquely identifies an article from all
  other articles published on the same day. Please note, however, that
  different articles on different days may have the same slug.
  <ul>
  <li>30other</li>
  <li>12reunion</li>
  </ul>
   - taxonomicClassifierList: This field specifies a list of taxonomic classifiers that place this
  article into a hierarchy of articles. The individual terms of each
  taxonomic classifier are separated with the '/' character.
  <br/>
  Examples Include:
  <ol>
  <li>Top/Features/Travel/Guides/Destinations/North America/United
  States/Arizona</li>
  <li>Top/News/U.S./Rockies</li>
  <li>Top/Opinion</li>
  </ol>
   - titleList: This field specifies a list of authored works that correspond to works
  mentioned in the article.
  <br/>
  Examples Include:
  <ol>
  <li>Greystoke: The Legend of Tarzan, Lord of the Apes (Movie)</li>
  <li>Law & Order (TV Program)</li>
  <li>BATTLEFIELD EARTH (BOOK)</li>
  </ol>
   - typesOfMaterialList: This field specifies a normalized list of terms describing the general
  editorial category of the article.
  <br />
  Examples Include:
  <ol>
  <li>REVIEW</li>
  <li>OBITUARY</li>
  <li>ANALYSIS</li>
  </ol>
   - url: This field specifies the location of the online version of the article. The
  &quot;Alternative Url&quot; field is preferred to this field on articles published
  on or after April 02, 2006, as the linked page will have richer content.
   - wordCount: This field specifies the number of words in the body of the article,
  including the lead paragraph.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'alternateURL', None, None, ), # 1
    (2, TType.STRING, 'articleAbstract', None, None, ), # 2
    (3, TType.STRING, 'authorBiography', None, None, ), # 3
    (4, TType.STRING, 'banner', None, None, ), # 4
    (5, TType.LIST, 'biographicalCategoryList', (TType.STRING,None), None, ), # 5
    (6, TType.STRING, 'columnName', None, None, ), # 6
    (7, TType.I32, 'columnNumber', None, None, ), # 7
    (8, TType.I64, 'correctionDate', None, None, ), # 8
    (9, TType.STRING, 'correctionText', None, None, ), # 9
    (10, TType.STRING, 'credit', None, None, ), # 10
    (11, TType.STRING, 'dayOfWeek', None, None, ), # 11
    (12, TType.LIST, 'descriptorList', (TType.STRING,None), None, ), # 12
    (13, TType.STRING, 'featurePage', None, None, ), # 13
    (14, TType.LIST, 'generalOnlineDescriptorList', (TType.STRING,None), None, ), # 14
    (15, TType.I32, 'guid', None, None, ), # 15
    (16, TType.STRING, 'kicker', None, None, ), # 16
    (17, TType.LIST, 'leadParagraphList', (TType.STRING,None), None, ), # 17
    (18, TType.LIST, 'locationList', (TType.STRING,None), None, ), # 18
    (19, TType.LIST, 'nameList', (TType.STRING,None), None, ), # 19
    (20, TType.STRING, 'newsDesk', None, None, ), # 20
    (21, TType.STRING, 'normalizedByline', None, None, ), # 21
    (22, TType.LIST, 'onlineDescriptorList', (TType.STRING,None), None, ), # 22
    (23, TType.STRING, 'onlineHeadline', None, None, ), # 23
    (24, TType.STRING, 'onlineLeadParagraph', None, None, ), # 24
    (25, TType.LIST, 'onlineLocationList', (TType.STRING,None), None, ), # 25
    (26, TType.LIST, 'onlineOrganizationList', (TType.STRING,None), None, ), # 26
    (27, TType.LIST, 'onlinePeople', (TType.STRING,None), None, ), # 27
    (28, TType.LIST, 'onlineSectionList', (TType.STRING,None), None, ), # 28
    (29, TType.LIST, 'onlineTitleList', (TType.STRING,None), None, ), # 29
    (30, TType.LIST, 'organizationList', (TType.STRING,None), None, ), # 30
    (31, TType.I32, 'page', None, None, ), # 31
    (32, TType.LIST, 'peopleList', (TType.STRING,None), None, ), # 32
    (33, TType.I64, 'publicationDate', None, None, ), # 33
    (34, TType.I32, 'publicationDayOfMonth', None, None, ), # 34
    (35, TType.I32, 'publicationMonth', None, None, ), # 35
    (36, TType.I32, 'publicationYear', None, None, ), # 36
    (37, TType.STRING, 'section', None, None, ), # 37
    (38, TType.STRING, 'seriesName', None, None, ), # 38
    (39, TType.STRING, 'slug', None, None, ), # 39
    (40, TType.LIST, 'taxonomicClassifierList', (TType.STRING,None), None, ), # 40
    (41, TType.LIST, 'titleList', (TType.STRING,None), None, ), # 41
    (42, TType.LIST, 'typesOfMaterialList', (TType.STRING,None), None, ), # 42
    (43, TType.STRING, 'url', None, None, ), # 43
    (44, TType.I32, 'wordCount', None, None, ), # 44
  )

  def __init__(self, alternateURL=None, articleAbstract=None, authorBiography=None, banner=None, biographicalCategoryList=None, columnName=None, columnNumber=None, correctionDate=None, correctionText=None, credit=None, dayOfWeek=None, descriptorList=None, featurePage=None, generalOnlineDescriptorList=None, guid=None, kicker=None, leadParagraphList=None, locationList=None, nameList=None, newsDesk=None, normalizedByline=None, onlineDescriptorList=None, onlineHeadline=None, onlineLeadParagraph=None, onlineLocationList=None, onlineOrganizationList=None, onlinePeople=None, onlineSectionList=None, onlineTitleList=None, organizationList=None, page=None, peopleList=None, publicationDate=None, publicationDayOfMonth=None, publicationMonth=None, publicationYear=None, section=None, seriesName=None, slug=None, taxonomicClassifierList=None, titleList=None, typesOfMaterialList=None, url=None, wordCount=None,):
    self.alternateURL = alternateURL
    self.articleAbstract = articleAbstract
    self.authorBiography = authorBiography
    self.banner = banner
    self.biographicalCategoryList = biographicalCategoryList
    self.columnName = columnName
    self.columnNumber = columnNumber
    self.correctionDate = correctionDate
    self.correctionText = correctionText
    self.credit = credit
    self.dayOfWeek = dayOfWeek
    self.descriptorList = descriptorList
    self.featurePage = featurePage
    self.generalOnlineDescriptorList = generalOnlineDescriptorList
    self.guid = guid
    self.kicker = kicker
    self.leadParagraphList = leadParagraphList
    self.locationList = locationList
    self.nameList = nameList
    self.newsDesk = newsDesk
    self.normalizedByline = normalizedByline
    self.onlineDescriptorList = onlineDescriptorList
    self.onlineHeadline = onlineHeadline
    self.onlineLeadParagraph = onlineLeadParagraph
    self.onlineLocationList = onlineLocationList
    self.onlineOrganizationList = onlineOrganizationList
    self.onlinePeople = onlinePeople
    self.onlineSectionList = onlineSectionList
    self.onlineTitleList = onlineTitleList
    self.organizationList = organizationList
    self.page = page
    self.peopleList = peopleList
    self.publicationDate = publicationDate
    self.publicationDayOfMonth = publicationDayOfMonth
    self.publicationMonth = publicationMonth
    self.publicationYear = publicationYear
    self.section = section
    self.seriesName = seriesName
    self.slug = slug
    self.taxonomicClassifierList = taxonomicClassifierList
    self.titleList = titleList
    self.typesOfMaterialList = typesOfMaterialList
    self.url = url
    self.wordCount = wordCount

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.alternateURL = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.articleAbstract = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.authorBiography = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.banner = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.biographicalCategoryList = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = iprot.readString().decode('utf-8')
            self.biographicalCategoryList.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.STRING:
          self.columnName = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.I32:
          self.columnNumber = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.I64:
          self.correctionDate = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 9:
        if ftype == TType.STRING:
          self.correctionText = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 10:
        if ftype == TType.STRING:
          self.credit = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 11:
        if ftype == TType.STRING:
          self.dayOfWeek = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 12:
        if ftype == TType.LIST:
          self.descriptorList = []
          (_etype9, _size6) = iprot.readListBegin()
          for _i10 in xrange(_size6):
            _elem11 = iprot.readString().decode('utf-8')
            self.descriptorList.append(_elem11)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 13:
        if ftype == TType.STRING:
          self.featurePage = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 14:
        if ftype == TType.LIST:
          self.generalOnlineDescriptorList = []
          (_etype15, _size12) = iprot.readListBegin()
          for _i16 in xrange(_size12):
            _elem17 = iprot.readString().decode('utf-8')
            self.generalOnlineDescriptorList.append(_elem17)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 15:
        if ftype == TType.I32:
          self.guid = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 16:
        if ftype == TType.STRING:
          self.kicker = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 17:
        if ftype == TType.LIST:
          self.leadParagraphList = []
          (_etype21, _size18) = iprot.readListBegin()
          for _i22 in xrange(_size18):
            _elem23 = iprot.readString().decode('utf-8')
            self.leadParagraphList.append(_elem23)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 18:
        if ftype == TType.LIST:
          self.locationList = []
          (_etype27, _size24) = iprot.readListBegin()
          for _i28 in xrange(_size24):
            _elem29 = iprot.readString().decode('utf-8')
            self.locationList.append(_elem29)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 19:
        if ftype == TType.LIST:
          self.nameList = []
          (_etype33, _size30) = iprot.readListBegin()
          for _i34 in xrange(_size30):
            _elem35 = iprot.readString().decode('utf-8')
            self.nameList.append(_elem35)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 20:
        if ftype == TType.STRING:
          self.newsDesk = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 21:
        if ftype == TType.STRING:
          self.normalizedByline = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 22:
        if ftype == TType.LIST:
          self.onlineDescriptorList = []
          (_etype39, _size36) = iprot.readListBegin()
          for _i40 in xrange(_size36):
            _elem41 = iprot.readString().decode('utf-8')
            self.onlineDescriptorList.append(_elem41)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 23:
        if ftype == TType.STRING:
          self.onlineHeadline = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 24:
        if ftype == TType.STRING:
          self.onlineLeadParagraph = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 25:
        if ftype == TType.LIST:
          self.onlineLocationList = []
          (_etype45, _size42) = iprot.readListBegin()
          for _i46 in xrange(_size42):
            _elem47 = iprot.readString().decode('utf-8')
            self.onlineLocationList.append(_elem47)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 26:
        if ftype == TType.LIST:
          self.onlineOrganizationList = []
          (_etype51, _size48) = iprot.readListBegin()
          for _i52 in xrange(_size48):
            _elem53 = iprot.readString().decode('utf-8')
            self.onlineOrganizationList.append(_elem53)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 27:
        if ftype == TType.LIST:
          self.onlinePeople = []
          (_etype57, _size54) = iprot.readListBegin()
          for _i58 in xrange(_size54):
            _elem59 = iprot.readString().decode('utf-8')
            self.onlinePeople.append(_elem59)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 28:
        if ftype == TType.LIST:
          self.onlineSectionList = []
          (_etype63, _size60) = iprot.readListBegin()
          for _i64 in xrange(_size60):
            _elem65 = iprot.readString().decode('utf-8')
            self.onlineSectionList.append(_elem65)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 29:
        if ftype == TType.LIST:
          self.onlineTitleList = []
          (_etype69, _size66) = iprot.readListBegin()
          for _i70 in xrange(_size66):
            _elem71 = iprot.readString().decode('utf-8')
            self.onlineTitleList.append(_elem71)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 30:
        if ftype == TType.LIST:
          self.organizationList = []
          (_etype75, _size72) = iprot.readListBegin()
          for _i76 in xrange(_size72):
            _elem77 = iprot.readString().decode('utf-8')
            self.organizationList.append(_elem77)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 31:
        if ftype == TType.I32:
          self.page = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 32:
        if ftype == TType.LIST:
          self.peopleList = []
          (_etype81, _size78) = iprot.readListBegin()
          for _i82 in xrange(_size78):
            _elem83 = iprot.readString().decode('utf-8')
            self.peopleList.append(_elem83)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 33:
        if ftype == TType.I64:
          self.publicationDate = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 34:
        if ftype == TType.I32:
          self.publicationDayOfMonth = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 35:
        if ftype == TType.I32:
          self.publicationMonth = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 36:
        if ftype == TType.I32:
          self.publicationYear = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 37:
        if ftype == TType.STRING:
          self.section = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 38:
        if ftype == TType.STRING:
          self.seriesName = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 39:
        if ftype == TType.STRING:
          self.slug = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 40:
        if ftype == TType.LIST:
          self.taxonomicClassifierList = []
          (_etype87, _size84) = iprot.readListBegin()
          for _i88 in xrange(_size84):
            _elem89 = iprot.readString().decode('utf-8')
            self.taxonomicClassifierList.append(_elem89)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 41:
        if ftype == TType.LIST:
          self.titleList = []
          (_etype93, _size90) = iprot.readListBegin()
          for _i94 in xrange(_size90):
            _elem95 = iprot.readString().decode('utf-8')
            self.titleList.append(_elem95)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 42:
        if ftype == TType.LIST:
          self.typesOfMaterialList = []
          (_etype99, _size96) = iprot.readListBegin()
          for _i100 in xrange(_size96):
            _elem101 = iprot.readString().decode('utf-8')
            self.typesOfMaterialList.append(_elem101)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 43:
        if ftype == TType.STRING:
          self.url = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 44:
        if ftype == TType.I32:
          self.wordCount = iprot.readI32();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('NITFInfo')
    if self.alternateURL is not None:
      oprot.writeFieldBegin('alternateURL', TType.STRING, 1)
      oprot.writeString(self.alternateURL.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.articleAbstract is not None:
      oprot.writeFieldBegin('articleAbstract', TType.STRING, 2)
      oprot.writeString(self.articleAbstract.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.authorBiography is not None:
      oprot.writeFieldBegin('authorBiography', TType.STRING, 3)
      oprot.writeString(self.authorBiography.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.banner is not None:
      oprot.writeFieldBegin('banner', TType.STRING, 4)
      oprot.writeString(self.banner.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.biographicalCategoryList is not None:
      oprot.writeFieldBegin('biographicalCategoryList', TType.LIST, 5)
      oprot.writeListBegin(TType.STRING, len(self.biographicalCategoryList))
      for iter102 in self.biographicalCategoryList:
        oprot.writeString(iter102.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.columnName is not None:
      oprot.writeFieldBegin('columnName', TType.STRING, 6)
      oprot.writeString(self.columnName.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.columnNumber is not None:
      oprot.writeFieldBegin('columnNumber', TType.I32, 7)
      oprot.writeI32(self.columnNumber)
      oprot.writeFieldEnd()
    if self.correctionDate is not None:
      oprot.writeFieldBegin('correctionDate', TType.I64, 8)
      oprot.writeI64(self.correctionDate)
      oprot.writeFieldEnd()
    if self.correctionText is not None:
      oprot.writeFieldBegin('correctionText', TType.STRING, 9)
      oprot.writeString(self.correctionText.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.credit is not None:
      oprot.writeFieldBegin('credit', TType.STRING, 10)
      oprot.writeString(self.credit.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.dayOfWeek is not None:
      oprot.writeFieldBegin('dayOfWeek', TType.STRING, 11)
      oprot.writeString(self.dayOfWeek.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.descriptorList is not None:
      oprot.writeFieldBegin('descriptorList', TType.LIST, 12)
      oprot.writeListBegin(TType.STRING, len(self.descriptorList))
      for iter103 in self.descriptorList:
        oprot.writeString(iter103.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.featurePage is not None:
      oprot.writeFieldBegin('featurePage', TType.STRING, 13)
      oprot.writeString(self.featurePage.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.generalOnlineDescriptorList is not None:
      oprot.writeFieldBegin('generalOnlineDescriptorList', TType.LIST, 14)
      oprot.writeListBegin(TType.STRING, len(self.generalOnlineDescriptorList))
      for iter104 in self.generalOnlineDescriptorList:
        oprot.writeString(iter104.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.guid is not None:
      oprot.writeFieldBegin('guid', TType.I32, 15)
      oprot.writeI32(self.guid)
      oprot.writeFieldEnd()
    if self.kicker is not None:
      oprot.writeFieldBegin('kicker', TType.STRING, 16)
      oprot.writeString(self.kicker.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.leadParagraphList is not None:
      oprot.writeFieldBegin('leadParagraphList', TType.LIST, 17)
      oprot.writeListBegin(TType.STRING, len(self.leadParagraphList))
      for iter105 in self.leadParagraphList:
        oprot.writeString(iter105.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.locationList is not None:
      oprot.writeFieldBegin('locationList', TType.LIST, 18)
      oprot.writeListBegin(TType.STRING, len(self.locationList))
      for iter106 in self.locationList:
        oprot.writeString(iter106.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.nameList is not None:
      oprot.writeFieldBegin('nameList', TType.LIST, 19)
      oprot.writeListBegin(TType.STRING, len(self.nameList))
      for iter107 in self.nameList:
        oprot.writeString(iter107.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.newsDesk is not None:
      oprot.writeFieldBegin('newsDesk', TType.STRING, 20)
      oprot.writeString(self.newsDesk.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.normalizedByline is not None:
      oprot.writeFieldBegin('normalizedByline', TType.STRING, 21)
      oprot.writeString(self.normalizedByline.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.onlineDescriptorList is not None:
      oprot.writeFieldBegin('onlineDescriptorList', TType.LIST, 22)
      oprot.writeListBegin(TType.STRING, len(self.onlineDescriptorList))
      for iter108 in self.onlineDescriptorList:
        oprot.writeString(iter108.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.onlineHeadline is not None:
      oprot.writeFieldBegin('onlineHeadline', TType.STRING, 23)
      oprot.writeString(self.onlineHeadline.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.onlineLeadParagraph is not None:
      oprot.writeFieldBegin('onlineLeadParagraph', TType.STRING, 24)
      oprot.writeString(self.onlineLeadParagraph.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.onlineLocationList is not None:
      oprot.writeFieldBegin('onlineLocationList', TType.LIST, 25)
      oprot.writeListBegin(TType.STRING, len(self.onlineLocationList))
      for iter109 in self.onlineLocationList:
        oprot.writeString(iter109.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.onlineOrganizationList is not None:
      oprot.writeFieldBegin('onlineOrganizationList', TType.LIST, 26)
      oprot.writeListBegin(TType.STRING, len(self.onlineOrganizationList))
      for iter110 in self.onlineOrganizationList:
        oprot.writeString(iter110.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.onlinePeople is not None:
      oprot.writeFieldBegin('onlinePeople', TType.LIST, 27)
      oprot.writeListBegin(TType.STRING, len(self.onlinePeople))
      for iter111 in self.onlinePeople:
        oprot.writeString(iter111.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.onlineSectionList is not None:
      oprot.writeFieldBegin('onlineSectionList', TType.LIST, 28)
      oprot.writeListBegin(TType.STRING, len(self.onlineSectionList))
      for iter112 in self.onlineSectionList:
        oprot.writeString(iter112.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.onlineTitleList is not None:
      oprot.writeFieldBegin('onlineTitleList', TType.LIST, 29)
      oprot.writeListBegin(TType.STRING, len(self.onlineTitleList))
      for iter113 in self.onlineTitleList:
        oprot.writeString(iter113.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.organizationList is not None:
      oprot.writeFieldBegin('organizationList', TType.LIST, 30)
      oprot.writeListBegin(TType.STRING, len(self.organizationList))
      for iter114 in self.organizationList:
        oprot.writeString(iter114.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.page is not None:
      oprot.writeFieldBegin('page', TType.I32, 31)
      oprot.writeI32(self.page)
      oprot.writeFieldEnd()
    if self.peopleList is not None:
      oprot.writeFieldBegin('peopleList', TType.LIST, 32)
      oprot.writeListBegin(TType.STRING, len(self.peopleList))
      for iter115 in self.peopleList:
        oprot.writeString(iter115.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.publicationDate is not None:
      oprot.writeFieldBegin('publicationDate', TType.I64, 33)
      oprot.writeI64(self.publicationDate)
      oprot.writeFieldEnd()
    if self.publicationDayOfMonth is not None:
      oprot.writeFieldBegin('publicationDayOfMonth', TType.I32, 34)
      oprot.writeI32(self.publicationDayOfMonth)
      oprot.writeFieldEnd()
    if self.publicationMonth is not None:
      oprot.writeFieldBegin('publicationMonth', TType.I32, 35)
      oprot.writeI32(self.publicationMonth)
      oprot.writeFieldEnd()
    if self.publicationYear is not None:
      oprot.writeFieldBegin('publicationYear', TType.I32, 36)
      oprot.writeI32(self.publicationYear)
      oprot.writeFieldEnd()
    if self.section is not None:
      oprot.writeFieldBegin('section', TType.STRING, 37)
      oprot.writeString(self.section.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.seriesName is not None:
      oprot.writeFieldBegin('seriesName', TType.STRING, 38)
      oprot.writeString(self.seriesName.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.slug is not None:
      oprot.writeFieldBegin('slug', TType.STRING, 39)
      oprot.writeString(self.slug.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.taxonomicClassifierList is not None:
      oprot.writeFieldBegin('taxonomicClassifierList', TType.LIST, 40)
      oprot.writeListBegin(TType.STRING, len(self.taxonomicClassifierList))
      for iter116 in self.taxonomicClassifierList:
        oprot.writeString(iter116.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.titleList is not None:
      oprot.writeFieldBegin('titleList', TType.LIST, 41)
      oprot.writeListBegin(TType.STRING, len(self.titleList))
      for iter117 in self.titleList:
        oprot.writeString(iter117.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.typesOfMaterialList is not None:
      oprot.writeFieldBegin('typesOfMaterialList', TType.LIST, 42)
      oprot.writeListBegin(TType.STRING, len(self.typesOfMaterialList))
      for iter118 in self.typesOfMaterialList:
        oprot.writeString(iter118.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.url is not None:
      oprot.writeFieldBegin('url', TType.STRING, 43)
      oprot.writeString(self.url.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.wordCount is not None:
      oprot.writeFieldBegin('wordCount', TType.I32, 44)
      oprot.writeI32(self.wordCount)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.alternateURL)
    value = (value * 31) ^ hash(self.articleAbstract)
    value = (value * 31) ^ hash(self.authorBiography)
    value = (value * 31) ^ hash(self.banner)
    value = (value * 31) ^ hash(self.biographicalCategoryList)
    value = (value * 31) ^ hash(self.columnName)
    value = (value * 31) ^ hash(self.columnNumber)
    value = (value * 31) ^ hash(self.correctionDate)
    value = (value * 31) ^ hash(self.correctionText)
    value = (value * 31) ^ hash(self.credit)
    value = (value * 31) ^ hash(self.dayOfWeek)
    value = (value * 31) ^ hash(self.descriptorList)
    value = (value * 31) ^ hash(self.featurePage)
    value = (value * 31) ^ hash(self.generalOnlineDescriptorList)
    value = (value * 31) ^ hash(self.guid)
    value = (value * 31) ^ hash(self.kicker)
    value = (value * 31) ^ hash(self.leadParagraphList)
    value = (value * 31) ^ hash(self.locationList)
    value = (value * 31) ^ hash(self.nameList)
    value = (value * 31) ^ hash(self.newsDesk)
    value = (value * 31) ^ hash(self.normalizedByline)
    value = (value * 31) ^ hash(self.onlineDescriptorList)
    value = (value * 31) ^ hash(self.onlineHeadline)
    value = (value * 31) ^ hash(self.onlineLeadParagraph)
    value = (value * 31) ^ hash(self.onlineLocationList)
    value = (value * 31) ^ hash(self.onlineOrganizationList)
    value = (value * 31) ^ hash(self.onlinePeople)
    value = (value * 31) ^ hash(self.onlineSectionList)
    value = (value * 31) ^ hash(self.onlineTitleList)
    value = (value * 31) ^ hash(self.organizationList)
    value = (value * 31) ^ hash(self.page)
    value = (value * 31) ^ hash(self.peopleList)
    value = (value * 31) ^ hash(self.publicationDate)
    value = (value * 31) ^ hash(self.publicationDayOfMonth)
    value = (value * 31) ^ hash(self.publicationMonth)
    value = (value * 31) ^ hash(self.publicationYear)
    value = (value * 31) ^ hash(self.section)
    value = (value * 31) ^ hash(self.seriesName)
    value = (value * 31) ^ hash(self.slug)
    value = (value * 31) ^ hash(self.taxonomicClassifierList)
    value = (value * 31) ^ hash(self.titleList)
    value = (value * 31) ^ hash(self.typesOfMaterialList)
    value = (value * 31) ^ hash(self.url)
    value = (value * 31) ^ hash(self.wordCount)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#!/usr/bin/env python

"""
"""

import unittest

from concrete import AnnotationMetadata, Communication
import concrete.inspect
from concrete.util import generate_UUID
from concrete.validate import validate_communication

from test_validate_communication import read_test_comm


class TestConcreteInspect(unittest.TestCase):
    def setUp(self):
        self.comm = read_test_comm()

    def test_print_conll_style_tags_for_communication(self):
        concrete.inspect.print_conll_style_tags_for_communication(
            self.comm, char_offsets=True, dependency=True, lemmas=True, ner=True, pos=True)

    def test_print_entities(self):
        concrete.inspect.print_entities(self.comm)

    def test_print_situation_mentions(self):
        concrete.inspect.print_situation_mentions(self.comm)

    def test_print_situations(self):
        concrete.inspect.print_situations(self.comm)

    def test_print_text_for_communication(self):
        concrete.inspect.print_text_for_communication(self.comm)

    def test_print_tokens_with_entityMentions(self):
        concrete.inspect.print_tokens_with_entityMentions(self.comm)

    def test_print_tokens_for_communication(self):
        concrete.inspect.print_tokens_for_communication(self.comm)

    def test_print_penn_treebank_for_communication(self):
        concrete.inspect.print_penn_treebank_for_communication(self.comm)


if __name__ == '__main__':
    unittest.main(buffer=True)
